import React, { useCallback, useEffect, useImperativeHandle, useRef } from 'react';
import { FormField, FormFieldSet, ChangeIndicatorProvider } from '@sanity/base/_unstable';
import { PatchEvent, set, unset, setIfMissing } from '@sanity/base/form';
import { Card, Select, Stack, TextInput } from '@sanity/ui';
import * as PathUtils from '@sanity/util/paths';
import AceEditor from 'react-ace';
import styled from 'styled-components';
import { useId } from '@reach/auto-id';
import createHighlightMarkers, { highlightMarkersCSS } from './createHighlightMarkers';
/* eslint-disable-next-line import/no-unassigned-import */
import './editorSupport';
import { ACE_EDITOR_PROPS, ACE_SET_OPTIONS, DEFAULT_THEME, LANGUAGE_ALIASES, PATH_CODE, PATH_FILENAME, PATH_LANGUAGE, SUPPORTED_LANGUAGES, SUPPORTED_THEMES, } from './config';
const EditorContainer = styled(Card) `
  position: relative;
  box-sizing: border-box;
  overflow: hidden;
  z-index: 0;

  .ace_editor {
    font-family: ${({ theme }) => theme.sanity.fonts.code.family};
    font-size: ${({ theme }) => theme.sanity.fonts.code.sizes[1]};
    line-height: inherit;
  }

  ${highlightMarkersCSS}

  &:not([disabled]):not([readonly]) {
    &:focus,
    &:focus-within {
      box-shadow: 0 0 0 2px ${({ theme }) => theme.sanity.color.base.focusRing};
      background-color: ${({ theme }) => theme.sanity.color.base.bg};
      border-color: ${({ theme }) => theme.sanity.color.base.focusRing};
    }
  }
`;
// Returns a string with the mode name if supported (because aliases), otherwise false
function isSupportedLanguage(mode) {
    const alias = LANGUAGE_ALIASES[mode];
    if (alias) {
        return alias;
    }
    const isSupported = SUPPORTED_LANGUAGES.find((lang) => lang.value === mode);
    if (isSupported) {
        return mode;
    }
    return false;
}
const CodeInput = React.forwardRef((props, ref) => {
    const aceEditorRef = useRef();
    const aceEditorId = useId();
    const { onFocus, onChange, onBlur, compareValue, value, presence, type, level = 0, readOnly, focusPath, } = props;
    useImperativeHandle(ref, () => ({
        focus: () => {
            aceEditorRef?.current?.editor?.focus();
        },
    }));
    const handleLanguageFocus = useCallback(() => {
        onFocus(PATH_LANGUAGE);
    }, [onFocus]);
    const handleCodeFocus = useCallback(() => {
        onFocus(PATH_CODE);
    }, [onFocus]);
    const handleFilenameFocus = useCallback(() => {
        onFocus(PATH_FILENAME);
    }, [onFocus]);
    const handleFilenameChange = useCallback((event) => {
        const val = event.target.value;
        const path = PATH_FILENAME;
        onChange(PatchEvent.from([setIfMissing({ _type: type.name }), val ? set(val, path) : unset(path)]));
    }, [onChange, type.name]);
    const getTheme = useCallback(() => {
        const preferredTheme = type.options?.theme;
        return preferredTheme && SUPPORTED_THEMES.find((theme) => theme === preferredTheme)
            ? preferredTheme
            : DEFAULT_THEME;
    }, [type]);
    const handleToggleSelectLine = useCallback((lineNumber) => {
        const editorSession = aceEditorRef.current?.editor?.getSession();
        const backgroundMarkers = editorSession?.getMarkers(true);
        const currentHighlightedLines = Object.keys(backgroundMarkers)
            .filter((key) => backgroundMarkers[key].type === 'screenLine')
            .map((key) => backgroundMarkers[key].range.start.row);
        const currentIndex = currentHighlightedLines.indexOf(lineNumber);
        if (currentIndex > -1) {
            // toggle remove
            currentHighlightedLines.splice(currentIndex, 1);
        }
        else {
            // toggle add
            currentHighlightedLines.push(lineNumber);
            currentHighlightedLines.sort();
        }
        onChange(PatchEvent.from(set(currentHighlightedLines.map((line) => 
        // ace starts at line (row) 0, but we store it starting at line 1
        line + 1), ['highlightedLines'])));
    }, [aceEditorRef, onChange]);
    const handleGutterMouseDown = useCallback((event) => {
        const target = event.domEvent.target;
        if (target.classList.contains('ace_gutter-cell')) {
            const row = event.getDocumentPosition().row;
            handleToggleSelectLine(row);
        }
    }, [handleToggleSelectLine]);
    useEffect(() => {
        const editor = aceEditorRef?.current?.editor;
        return () => {
            editor?.session?.removeListener('guttermousedown', handleGutterMouseDown);
        };
    }, [aceEditorRef, handleGutterMouseDown]);
    const handleEditorLoad = useCallback((editor) => {
        editor?.on('guttermousedown', handleGutterMouseDown);
    }, [handleGutterMouseDown]);
    const getLanguageAlternatives = useCallback(() => {
        const languageAlternatives = type.options?.languageAlternatives;
        if (!languageAlternatives) {
            return SUPPORTED_LANGUAGES;
        }
        if (!Array.isArray(languageAlternatives)) {
            throw new Error(`'options.languageAlternatives' should be an array, got ${typeof languageAlternatives}`);
        }
        return languageAlternatives.reduce((acc, { title, value: val, mode }) => {
            const alias = LANGUAGE_ALIASES[val];
            if (alias) {
                // eslint-disable-next-line no-console
                console.warn(`'options.languageAlternatives' lists a language with value "%s", which is an alias of "%s" - please replace the value to read "%s"`, val, alias, alias);
                return acc.concat({ title, value: alias, mode: mode });
            }
            if (!mode && !SUPPORTED_LANGUAGES.find((lang) => lang.value === val)) {
                // eslint-disable-next-line no-console
                console.warn(`'options.languageAlternatives' lists a language which is not supported: "%s", syntax highlighting will be disabled.`, val);
            }
            return acc.concat({ title, value: val, mode });
        }, []);
    }, [type]);
    const handleCodeChange = useCallback((code) => {
        const path = PATH_CODE;
        const fixedLanguage = type.options?.language;
        onChange(PatchEvent.from([
            setIfMissing({ _type: type.name, language: fixedLanguage }),
            code ? set(code, path) : unset(path),
        ]));
    }, [onChange, type]);
    const handleLanguageChange = useCallback((event) => {
        const val = event.currentTarget.value;
        const path = PATH_LANGUAGE;
        onChange(PatchEvent.from([setIfMissing({ _type: type.name }), val ? set(val, path) : unset(path)]));
    }, [onChange, type.name]);
    const languages = getLanguageAlternatives().slice();
    const selectedLanguage = props?.value?.language
        ? languages.find((item) => item.value === props?.value?.language)
        : undefined;
    const languageField = type.fields.find((field) => field.name === 'language');
    const filenameField = type.fields.find((field) => field.name === 'filename');
    const languageCompareValue = PathUtils.get(compareValue, PATH_LANGUAGE);
    const codeCompareValue = PathUtils.get(compareValue, PATH_CODE);
    const filenameCompareValue = PathUtils.get(compareValue, PATH_FILENAME);
    const languagePresence = presence.filter((presenceItem) => PathUtils.startsWith(PATH_LANGUAGE, presenceItem.path));
    const codePresence = presence.filter((presenceItem) => PathUtils.startsWith(PATH_CODE, presenceItem.path));
    const filenamePresence = presence.filter((presenceItem) => PathUtils.startsWith(PATH_FILENAME, presenceItem.path));
    const renderEditor = useCallback(() => {
        const fixedLanguage = type.options?.language;
        const language = value?.language || fixedLanguage;
        // the language config from the schema
        const configured = languages.find((entry) => entry.value === language);
        // is the language officially supported (e.g. we import the mode by default)
        const supported = language && isSupportedLanguage(language);
        const mode = configured?.mode || (supported ? language : 'text');
        return (React.createElement(EditorContainer, { radius: 1, shadow: 1, readOnly: readOnly },
            React.createElement(AceEditor, { ref: aceEditorRef, mode: mode, theme: getTheme(), width: "100%", onChange: handleCodeChange, name: `editor-${aceEditorId}`, value: (value && value.code) || '', markers: value && value.highlightedLines
                    ? createHighlightMarkers(value.highlightedLines)
                    : undefined, onLoad: handleEditorLoad, readOnly: readOnly, tabSize: 2, wrapEnabled: true, setOptions: ACE_SET_OPTIONS, editorProps: ACE_EDITOR_PROPS, onFocus: handleCodeFocus, onBlur: onBlur })));
    }, [
        aceEditorId,
        getTheme,
        handleCodeChange,
        handleCodeFocus,
        handleEditorLoad,
        languages,
        onBlur,
        readOnly,
        type,
        value,
    ]);
    if (type.options?.language) {
        return (React.createElement(ChangeIndicatorProvider, { path: PATH_CODE, focusPath: focusPath, value: value?.code, compareValue: codeCompareValue },
            React.createElement(FormFieldSet, { title: type.title, description: type.description, level: level }, renderEditor())));
    }
    return (React.createElement(FormFieldSet, { title: type.title, description: type.description, level: level, __unstable_changeIndicator: false },
        React.createElement(Stack, { space: 3 },
            React.createElement(ChangeIndicatorProvider, { path: PATH_LANGUAGE, focusPath: focusPath, value: selectedLanguage?.value, compareValue: languageCompareValue },
                React.createElement(FormField, { level: level + 1, label: languageField?.type.title || 'Language', __unstable_presence: languagePresence },
                    React.createElement(Select, { onChange: handleLanguageChange, readOnly: readOnly, value: selectedLanguage?.value || '', onFocus: handleLanguageFocus, onBlur: onBlur }, languages.map((lang) => (React.createElement("option", { key: lang.value, value: lang.value }, lang.title)))))),
            type.options?.withFilename && (React.createElement(ChangeIndicatorProvider, { path: PATH_FILENAME, focusPath: focusPath, value: value?.filename, compareValue: filenameCompareValue },
                React.createElement(FormField, { label: filenameField?.type.title || 'Filename', level: level + 1, __unstable_presence: filenamePresence },
                    React.createElement(TextInput, { name: "filename", value: value?.filename || '', placeholder: filenameField?.type.placeholder, onChange: handleFilenameChange, onFocus: handleFilenameFocus, onBlur: onBlur })))),
            React.createElement(ChangeIndicatorProvider, { path: PATH_CODE, focusPath: focusPath, value: value?.code, compareValue: codeCompareValue },
                React.createElement(FormField, { label: "Code", level: level + 1, __unstable_presence: codePresence }, renderEditor())))));
});
CodeInput.displayName = 'CodeInput';
export default CodeInput;
//# sourceMappingURL=CodeInput.js.map